@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: metar.cxx,v 1.14.2.4 2007-10-13 14:05:48 durk Exp $
+// $Id$
 
 /**
  * @file metar.cxx
@@ -206,7 +206,7 @@ char *SGMetar::loadData(const char *id, const string& proxy, const string& port,
 	if (!proxy.empty())
 		get += "http://weather.noaa.gov";
 
-	snprintf(buf, (buflen*2), "%ld", time);
+	sprintf(buf, "%ld", time);
 	get += path + " HTTP/1.0\015\012X-Time: " + buf + "\015\012";
 
 	if (!auth.empty())
@@ -493,7 +493,6 @@ bool SGMetar::scanVisibility()
 		distance = i;
 	} else {
 // M?(\d{1,2}|\d{1,2}/\d{1,2}|\d{1,2} \d{1,2}/\d{1,2})(SM|KM)
-		modifier = 0;
 		if (*m == 'M')
 			m++, modifier = SGMetarVisibility::LESS_THAN;
 
@@ -1182,7 +1181,7 @@ const struct Token *SGMetar::scanToken(char **str, const struct Token *list)
 {
 	const struct Token *longest = 0;
 	int maxlen = 0, len;
-	char *s;
+	const char *s;
 	for (int i = 0; (s = list[i].id); i++) {
 		len = strlen(s);
 		if (!strncmp(s, *str, len) && len > maxlen) {
