@@ -38,7 +38,7 @@
 
 #include <vector>
 
-SG_USING_STD(vector);
+using std::vector;
 
 
 typedef struct {
@@ -189,6 +189,8 @@ SGEnviro::SGEnviro() :
 }
 
 SGEnviro::~SGEnviro(void) {
+  // OSGFIXME
+  return;
 	list_of_lightning::iterator iLightning;
 	for( iLightning = lightnings.begin() ; iLightning != lightnings.end() ; iLightning++ ) {
 		delete (*iLightning);
@@ -197,10 +199,10 @@ SGEnviro::~SGEnviro(void) {
 }
 
 void SGEnviro::startOfFrame( sgVec3 p, sgVec3 up, double lon, double lat, double alt, double delta_time) {
+  // OSGFIXME
+  return;
 	view_in_cloud = false;
 	// ask the impostor cache to do some cleanup
-	if(SGNewCloud::cldCache)
-		SGNewCloud::cldCache->startNewFrame();
 	last_cloud_turbulence = cloud_turbulence;
 	cloud_turbulence = 0.0;
 	elapsed_time += delta_time;
@@ -251,43 +253,10 @@ void SGEnviro::set_view_in_cloud(bool incloud) {
 	view_in_cloud = incloud;
 }
 
-int SGEnviro::get_CacheResolution(void) const {
-	return SGCloudField::get_CacheResolution();
-}
-
-int SGEnviro::get_clouds_CacheSize(void) const {
-	return SGCloudField::get_CacheSize();
-}
-float SGEnviro::get_clouds_visibility(void) const {
-	return SGCloudField::get_CloudVis();
-}
-float SGEnviro::get_clouds_density(void) const {
-	return SGCloudField::get_density();
-}
-bool SGEnviro::get_clouds_enable_state(void) const {
-	return SGCloudField::get_enable3dClouds();
-}
-
 bool SGEnviro::get_turbulence_enable_state(void) const {
 	return turbulence_enable_state;
 }
 
-void SGEnviro::set_CacheResolution(int resolutionPixels) {
-	SGCloudField::set_CacheResolution(resolutionPixels);
-}
-
-void SGEnviro::set_clouds_CacheSize(int sizeKb) {
-	SGCloudField::set_CacheSize(sizeKb);
-}
-void SGEnviro::set_clouds_visibility(float distance) {
-	SGCloudField::set_CloudVis(distance);
-}
-void SGEnviro::set_clouds_density(float density) {
-	SGCloudField::set_density(density);
-}
-void SGEnviro::set_clouds_enable_state(bool enable) {
-	SGCloudField::set_enable3dClouds(enable);
-}
 void SGEnviro::set_turbulence_enable_state(bool enable) {
 	turbulence_enable_state = enable;
 }
@@ -319,12 +288,14 @@ void SGEnviro::set_lightning_enable_state(bool enable) {
 }
 
 void SGEnviro::setLight(sgVec4 adj_fog_color) {
+  // OSGFIXME
+  return;
 	sgCopyVec4( fog_color, adj_fog_color );
 	if( false ) {
 	//    ssgGetLight( 0 ) -> setColour( GL_DIFFUSE, l->scene_diffuse() );
 	}
 }
-
+#if 0
 void SGEnviro::callback_cloud(float heading, float alt, float radius, int family, float dist, int cloudId) {
 	// send data to wx radar
 	// compute turbulence
@@ -435,12 +406,16 @@ void SGEnviro::callback_cloud(float heading, float alt, float radius, int family
 		}
 }
 
+#endif
+
 list_of_SGWxRadarEcho *SGEnviro::get_radar_echo(void) {
 	return &radarEcho;
 }
 
 // precipitation rendering code
 void SGEnviro::DrawCone2(float baseRadius, float height, int slices, bool down, double rain_norm, double speed) {
+  // OSGFIXME
+  return;
 	sgVec3 light;
 	sgAddVec3( light, fog_color, min_light );
 	float da = SG_PI * 2.0f / (float) slices;
@@ -489,6 +464,8 @@ void SGEnviro::DrawCone2(float baseRadius, float height, int slices, bool down,
 }
 
 void SGEnviro::drawRain(double pitch, double roll, double heading, double hspeed, double rain_norm) {
+  // OSGFIXME
+  return;
 
 #if 0
 	static int debug_period = 0;
@@ -558,6 +535,8 @@ void SGEnviro::set_soundMgr(SGSoundMgr *mgr) {
 }
 
 void SGEnviro::drawPrecipitation(double rain_norm, double snow_norm, double hail_norm, double pitch, double roll, double heading, double hspeed) {
+  // OSGFIXME
+  return;
 	if( precipitation_enable_state && rain_norm > 0.0)
 	  if( precipitation_max_alt >= last_alt )
 		drawRain(pitch, roll, heading, hspeed, rain_norm);
@@ -580,6 +559,8 @@ SGLightning::~SGLightning() {
 
 // lightning rendering code
 void SGLightning::lt_build_tree_branch(int tree_nr, Point3D &start, float energy, int nbseg, float segsize) {
+  // OSGFIXME
+  return;
 
 	sgVec3 dir, newdir;
 	int nseg = 0;
@@ -627,6 +608,8 @@ void SGLightning::lt_build_tree_branch(int tree_nr, Point3D &start, float energy
 }
 
 void SGLightning::lt_build(void) {
+  // OSGFIXME
+  return;
     Point3D top;
     nb_tree = 0;
     top[PX] = 0 ;
@@ -652,6 +635,8 @@ void SGLightning::lt_build(void) {
 
 
 void SGLightning::lt_Render(void) {
+  // OSGFIXME
+  return;
 	float flash = 0.5;
 	if( fmod(sgEnviro.elapsed_time*100.0, 100.0) > 50.0 )
 		flash = sg_random() * 0.75f + 0.25f;
@@ -676,10 +661,12 @@ void SGLightning::lt_Render(void) {
 	glDisable( GL_FOG );
 	glPushMatrix();
 	sgMat4 modelview, tmp;
-    ssgGetModelviewMatrix( modelview );
+    // OSGFIXME
+//     ssgGetModelviewMatrix( modelview );
 	sgCopyMat4( tmp, sgEnviro.transform );
     sgPostMultMat4( tmp, modelview );
-    ssgLoadModelviewMatrix( tmp );
+    // OSGFIXME
+//     ssgLoadModelviewMatrix( tmp );
 
     Point3D start( sgEnviro.last_lon*SG_DEGREES_TO_RADIANS, sgEnviro.last_lat*SG_DEGREES_TO_RADIANS, 0.0 );
     Point3D dest( lon*SG_DEGREES_TO_RADIANS, lat*SG_DEGREES_TO_RADIANS, 0.0 );
@@ -737,6 +724,8 @@ void SGLightning::lt_Render(void) {
 }
 
 void SGEnviro::addLightning(double lon, double lat, double alt) {
+  // OSGFIXME
+  return;
 	if( lightnings.size() > 10)
 		return;
 	SGLightning *lt= new SGLightning(lon, lat, alt);
@@ -744,6 +733,8 @@ void SGEnviro::addLightning(double lon, double lat, double alt) {
 }
 
 void SGEnviro::drawLightning(void) {
+  // OSGFIXME
+  return;
 	list_of_lightning::iterator iLightning;
 	// play 'thunder' for lightning
 	if( snd_active )
