@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: sg_binobj.cxx,v 1.16.2.1 2007-02-05 21:41:33 mfranz Exp $
+// $Id$
 //
 
 
@@ -33,7 +33,8 @@
 #include <time.h>
 
 #include <vector>
-#include STL_STRING
+#include <string>
+#include <iostream>
 
 #include <simgear/bucket/newbucket.hxx>
 #include <simgear/misc/sg_path.hxx>
@@ -42,9 +43,10 @@
 #include "sg_binobj.hxx"
 
 
-SG_USING_STD( string );
-SG_USING_STD( vector );
-
+using std::string;
+using std::vector;
+using std::cout;
+using std::endl;
 
 enum sgObjectTypes {
     SG_BOUNDING_SPHERE = 0,
@@ -174,7 +176,7 @@ static void read_object( gzFile fp,
     int idx_size;
     bool do_vertices, do_normals, do_colors, do_texcoords;
     int j, k, idx;
-    static sgSimpleBuffer buf( 32768 );  // 32 Kb
+    sgSimpleBuffer buf( 32768 );  // 32 Kb
     char material[256];
 
     // default values
@@ -280,13 +282,13 @@ static void read_object( gzFile fp,
 
 // read a binary file and populate the provided structures.
 bool SGBinObject::read_bin( const string& file ) {
-    Point3D p;
+    SGVec3d p;
     int i, j, k;
     unsigned int nbytes;
-    static sgSimpleBuffer buf( 32768 );  // 32 Kb
+    sgSimpleBuffer buf( 32768 );  // 32 Kb
 
     // zero out structures
-    gbs_center = Point3D( 0 );
+    gbs_center = SGVec3d(0, 0, 0);
     gbs_radius = 0.0;
 
     wgs84_nodes.clear();
@@ -364,18 +366,32 @@ bool SGBinObject::read_bin( const string& file ) {
 #endif
 
     // read number of top level objects
-    short nobjects;
-    sgReadShort( fp, &nobjects );
+    unsigned short nobjects;
+    if ( version >= 7 ) {
+        sgReadUShort( fp, &nobjects );
+    } else {
+        short tmp;
+        sgReadShort( fp, &tmp );
+        nobjects = tmp;
+    }
     // cout << "Total objects to read = " << nobjects << endl;
 
     // read in objects
     for ( i = 0; i < nobjects; ++i ) {
 	// read object header
 	char obj_type;
-	short nproperties, nelements;
+	unsigned short nproperties, nelements;
 	sgReadChar( fp, &obj_type );
-	sgReadShort( fp, &nproperties );
-	sgReadShort( fp, &nelements );
+        if ( version >= 7 ) {
+            sgReadUShort( fp, &nproperties );
+            sgReadUShort( fp, &nelements );
+        } else {
+            short tmp;
+            sgReadShort( fp, &tmp );
+            nproperties = tmp;
+            sgReadShort( fp, &tmp );
+            nelements = tmp;
+        }
 
 	// cout << "object " << i << " = " << (int)obj_type << " props = "
 	//      << nproperties << " elements = " << nelements << endl;
@@ -407,7 +423,7 @@ bool SGBinObject::read_bin( const string& file ) {
 		    sgEndianSwap( (uint64_t *)&(dptr[1]) );
 		    sgEndianSwap( (uint64_t *)&(dptr[2]) );
 		}
-		gbs_center = Point3D( dptr[0], dptr[1], dptr[2] );
+		gbs_center = SGVec3d( dptr[0], dptr[1], dptr[2] );
 		// cout << "Center = " << gbs_center << endl;
 		ptr += sizeof(double) * 3;
 		
@@ -447,7 +463,7 @@ bool SGBinObject::read_bin( const string& file ) {
 			sgEndianSwap( (uint32_t *)&(fptr[1]) );
 			sgEndianSwap( (uint32_t *)&(fptr[2]) );
 		    }
-		    wgs84_nodes.push_back( Point3D(fptr[0], fptr[1], fptr[2]) );
+		    wgs84_nodes.push_back( SGVec3d(fptr[0], fptr[1], fptr[2]) );
 		    fptr += 3;
 		}
 	    }
@@ -481,7 +497,8 @@ bool SGBinObject::read_bin( const string& file ) {
 			sgEndianSwap( (uint32_t *)&(fptr[2]) );
 			sgEndianSwap( (uint32_t *)&(fptr[3]) );
 		    }
-		    colors.push_back( Point3D( fptr[0], fptr[1], fptr[2] ) );
+		    SGVec4f color( fptr[0], fptr[1], fptr[2], fptr[3] );
+		    colors.push_back( color );
 		    fptr += 4;
 		}
 	    }
@@ -508,14 +525,11 @@ bool SGBinObject::read_bin( const string& file ) {
 		int count = nbytes / 3;
 		normals.reserve( count );
 		for ( k = 0; k < count; ++k ) {
-                    sgdVec3 normal;
-                    sgdSetVec3( normal,
-                               (ptr[0]) / 127.5 - 1.0,
-                               (ptr[1]) / 127.5 - 1.0,
-                               (ptr[2]) / 127.5 - 1.0 );
-                    sgdNormalizeVec3( normal );
+                    SGVec3f normal((ptr[0]) / 127.5 - 1.0,
+                                   (ptr[1]) / 127.5 - 1.0,
+                                   (ptr[2]) / 127.5 - 1.0);
 
-		    normals.push_back(Point3D(normal[0], normal[1], normal[2]));
+		    normals.push_back(normalize(normal));
 		    ptr += 3;
 		}
 	    }
@@ -547,7 +561,7 @@ bool SGBinObject::read_bin( const string& file ) {
 			sgEndianSwap( (uint32_t *)&(fptr[0]) );
 			sgEndianSwap( (uint32_t *)&(fptr[1]) );
 		    }
-		    texcoords.push_back( Point3D( fptr[0], fptr[1], 0 ) );
+		    texcoords.push_back( SGVec2f( fptr[0], fptr[1] ) );
 		    fptr += 2;
 		}
 	    }
@@ -612,10 +626,6 @@ bool SGBinObject::read_bin( const string& file ) {
 bool SGBinObject::write_bin( const string& base, const string& name,
 			     const SGBucket& b )
 {
-    Point3D p;
-    sgVec2 t;
-    sgVec3 pt;
-    sgVec4 color;
     int i, j;
     unsigned char idx_mask;
     int idx_size;
@@ -655,7 +665,7 @@ bool SGBinObject::write_bin( const string& base, const string& name,
     string material;
     int start;
     int end;
-    short nobjects = 0;
+    unsigned short nobjects = 0;
     nobjects++;			// for gbs
     nobjects++;			// for vertices
     nobjects++;			// for colors
@@ -663,7 +673,7 @@ bool SGBinObject::write_bin( const string& base, const string& name,
     nobjects++;			// for texcoords
 
     // points
-    short npts = 0;
+    unsigned short npts = 0;
     start = 0; end = 1;
     while ( start < (int)pt_materials.size() ) {
 	material = pt_materials[start];
@@ -677,7 +687,7 @@ bool SGBinObject::write_bin( const string& base, const string& name,
     nobjects += npts;
 
     // tris
-    short ntris = 0;
+    unsigned short ntris = 0;
     start = 0; end = 1;
     while ( start < (int)tri_materials.size() ) {
 	material = tri_materials[start];
@@ -691,7 +701,7 @@ bool SGBinObject::write_bin( const string& base, const string& name,
     nobjects += ntris;
 
     // strips
-    short nstrips = 0;
+    unsigned short nstrips = 0;
     start = 0; end = 1;
     while ( start < (int)strip_materials.size() ) {
 	material = strip_materials[start];
@@ -705,7 +715,7 @@ bool SGBinObject::write_bin( const string& base, const string& name,
     nobjects += nstrips;
 
     // fans
-    short nfans = 0;
+    unsigned short nfans = 0;
     start = 0; end = 1;
     while ( start < (int)fan_materials.size() ) {
 	material = fan_materials[start];
@@ -719,12 +729,12 @@ bool SGBinObject::write_bin( const string& base, const string& name,
     nobjects += nfans;
 
     cout << "total top level objects = " << nobjects << endl;
-    sgWriteShort( fp, nobjects );
+    sgWriteUShort( fp, nobjects );
 
     // write bounding sphere
     sgWriteChar( fp, (char)SG_BOUNDING_SPHERE );        // type
-    sgWriteShort( fp, 0 );		                // nproperties
-    sgWriteShort( fp, 1 );		                // nelements
+    sgWriteUShort( fp, 0 );		                // nproperties
+    sgWriteUShort( fp, 1 );		                // nelements
 
     sgWriteUInt( fp, sizeof(double) * 3 + sizeof(float) ); // nbytes
     sgdVec3 center;
@@ -734,37 +744,31 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 
     // dump vertex list
     sgWriteChar( fp, (char)SG_VERTEX_LIST );             // type
-    sgWriteShort( fp, 0 );		                 // nproperties
-    sgWriteShort( fp, 1 );		                 // nelements
+    sgWriteUShort( fp, 0 );		                 // nproperties
+    sgWriteUShort( fp, 1 );		                 // nelements
     sgWriteUInt( fp, wgs84_nodes.size() * sizeof(float) * 3 ); // nbytes
     for ( i = 0; i < (int)wgs84_nodes.size(); ++i ) {
-	p = wgs84_nodes[i] - gbs_center;
-	sgSetVec3( pt, p.x(), p.y(), p.z() );
-	sgWriteVec3( fp, pt );
+        SGVec3f p = toVec3f(wgs84_nodes[i] - gbs_center);
+	sgWriteVec3( fp, p.data() );
     }
 
     // dump vertex color list
     sgWriteChar( fp, (char)SG_COLOR_LIST );             // type
-    sgWriteShort( fp, 0 );		                 // nproperties
-    sgWriteShort( fp, 1 );		                 // nelements
+    sgWriteUShort( fp, 0 );		                 // nproperties
+    sgWriteUShort( fp, 1 );		                 // nelements
     sgWriteUInt( fp, colors.size() * sizeof(float) * 4 ); // nbytes
     for ( i = 0; i < (int)colors.size(); ++i ) {
-	p = colors[i];
-        // Right now we have a place holder for color alpha but we
-        // need to update the interface so the calling program can
-        // provide the info.
-	sgSetVec4( color, p.x(), p.y(), p.z(), 1.0 );
-	sgWriteVec4( fp, color );
+	sgWriteVec4( fp, colors[i].data() );
     }
 
     // dump vertex normal list
     sgWriteChar( fp, (char)SG_NORMAL_LIST );            // type
-    sgWriteShort( fp, 0 );		                // nproperties
-    sgWriteShort( fp, 1 );		                // nelements
+    sgWriteUShort( fp, 0 );		                // nproperties
+    sgWriteUShort( fp, 1 );		                // nelements
     sgWriteUInt( fp, normals.size() * 3 );              // nbytes
     char normal[3];
     for ( i = 0; i < (int)normals.size(); ++i ) {
-	p = normals[i];
+        SGVec3f p = normals[i];
 	normal[0] = (unsigned char)((p.x() + 1.0) * 127.5);
 	normal[1] = (unsigned char)((p.y() + 1.0) * 127.5);
 	normal[2] = (unsigned char)((p.z() + 1.0) * 127.5);
@@ -773,13 +777,11 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 
     // dump texture coordinates
     sgWriteChar( fp, (char)SG_TEXCOORD_LIST );          // type
-    sgWriteShort( fp, 0 );		                // nproperties
-    sgWriteShort( fp, 1 );		                // nelements
+    sgWriteUShort( fp, 0 );		                // nproperties
+    sgWriteUShort( fp, 1 );		                // nelements
     sgWriteUInt( fp, texcoords.size() * sizeof(float) * 2 ); // nbytes
     for ( i = 0; i < (int)texcoords.size(); ++i ) {
-	p = texcoords[i];
-	sgSetVec2( t, p.x(), p.y() );
-	sgWriteVec2( fp, t );
+	sgWriteVec2( fp, texcoords[i].data() );
     }
 
     // dump point groups if they exist
@@ -800,8 +802,8 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 
 	    // write group headers
 	    sgWriteChar( fp, (char)SG_POINTS );          // type
-	    sgWriteShort( fp, 2 );		         // nproperties
-	    sgWriteShort( fp, end - start );             // nelements
+	    sgWriteUShort( fp, 2 );		         // nproperties
+	    sgWriteUShort( fp, end - start );             // nelements
 
 	    sgWriteChar( fp, (char)SG_MATERIAL );        // property
 	    sgWriteUInt( fp, material.length() );        // nbytes
@@ -820,19 +822,20 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 	    // write strips
 	    for ( i = start; i < end; ++i ) {
 		// nbytes
-		sgWriteUInt( fp, pts_v[i].size() * idx_size * sizeof(short) );
+		sgWriteUInt( fp, pts_v[i].size() * idx_size
+                                 * sizeof(unsigned short) );
 		for ( j = 0; j < (int)pts_v[i].size(); ++j ) {
 		    if ( pts_v.size() ) { 
-			sgWriteShort( fp, (short)pts_v[i][j] );
+			sgWriteUShort( fp, (unsigned short)pts_v[i][j] );
 		    }
 		    if ( pts_n.size() ) { 
-			sgWriteShort( fp, (short)pts_n[i][j] );
+			sgWriteUShort( fp, (unsigned short)pts_n[i][j] );
 		    }
 		    if ( pts_c.size() ) { 
-			sgWriteShort( fp, (short)pts_c[i][j] );
+			sgWriteUShort( fp, (unsigned short)pts_c[i][j] );
 		    }
 		    if ( pts_tc.size() ) { 
-			sgWriteShort( fp, (short)pts_tc[i][j] );
+			sgWriteUShort( fp, (unsigned short)pts_tc[i][j] );
 		    }
 		}
 	    }
@@ -861,8 +864,8 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 
 	    // write group headers
 	    sgWriteChar( fp, (char)SG_TRIANGLE_FACES ); // type
-	    sgWriteShort( fp, 2 );		        // nproperties
-	    sgWriteShort( fp, 1 );                      // nelements
+	    sgWriteUShort( fp, 2 );		        // nproperties
+	    sgWriteUShort( fp, 1 );                      // nelements
 
 	    sgWriteChar( fp, (char)SG_MATERIAL );       // property
 	    sgWriteUInt( fp, material.length() );        // nbytes
@@ -879,22 +882,23 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 	    sgWriteChar( fp, idx_mask );
 
 	    // nbytes
-	    sgWriteUInt( fp, (end - start) * 3 * idx_size * sizeof(short) );
+	    sgWriteUInt( fp, (end - start) * 3 * idx_size
+                             * sizeof(unsigned short) );
 
 	    // write group
 	    for ( i = start; i < end; ++i ) {
 		for ( j = 0; j < 3; ++j ) {
 		    if ( tris_v.size() ) {
-			sgWriteShort( fp, (short)tris_v[i][j] );
+			sgWriteUShort( fp, (unsigned short)tris_v[i][j] );
 		    }
 		    if ( tris_n.size() ) {
-			sgWriteShort( fp, (short)tris_n[i][j] );
+			sgWriteUShort( fp, (unsigned short)tris_n[i][j] );
 		    }
 		    if ( tris_c.size() ) {
-			sgWriteShort( fp, (short)tris_c[i][j] );
+			sgWriteUShort( fp, (unsigned short)tris_c[i][j] );
 		    }
 		    if ( tris_tc.size() ) {
-			sgWriteShort( fp, (short)tris_tc[i][j] );
+			sgWriteUShort( fp, (unsigned short)tris_tc[i][j] );
 		    }
 		}
 	    }
@@ -922,8 +926,8 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 
 	    // write group headers
 	    sgWriteChar( fp, (char)SG_TRIANGLE_STRIPS ); // type
-	    sgWriteShort( fp, 2 );		         // nproperties
-	    sgWriteShort( fp, end - start );             // nelements
+	    sgWriteUShort( fp, 2 );		         // nproperties
+	    sgWriteUShort( fp, end - start );             // nelements
 
 	    sgWriteChar( fp, (char)SG_MATERIAL );        // property
 	    sgWriteUInt( fp, material.length() );        // nbytes
@@ -942,19 +946,20 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 	    // write strips
 	    for ( i = start; i < end; ++i ) {
 		// nbytes
-		sgWriteUInt( fp, strips_v[i].size() * idx_size * sizeof(short));
+		sgWriteUInt( fp, strips_v[i].size() * idx_size
+                                 * sizeof(unsigned short));
 		for ( j = 0; j < (int)strips_v[i].size(); ++j ) {
 		    if ( strips_v.size() ) { 
-			sgWriteShort( fp, (short)strips_v[i][j] );
+			sgWriteUShort( fp, (unsigned short)strips_v[i][j] );
 		    }
 		    if ( strips_n.size() ) { 
-			sgWriteShort( fp, (short)strips_n[i][j] );
+			sgWriteUShort( fp, (unsigned short)strips_n[i][j] );
 		    }
 		    if ( strips_c.size() ) { 
-			sgWriteShort( fp, (short)strips_c[i][j] );
+			sgWriteUShort( fp, (unsigned short)strips_c[i][j] );
 		    }
 		    if ( strips_tc.size() ) { 
-			sgWriteShort( fp, (short)strips_tc[i][j] );
+			sgWriteUShort( fp, (unsigned short)strips_tc[i][j] );
 		    }
 		}
 	    }
@@ -982,8 +987,8 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 
 	    // write group headers
 	    sgWriteChar( fp, (char)SG_TRIANGLE_FANS );   // type
-	    sgWriteShort( fp, 2 );		         // nproperties
-	    sgWriteShort( fp, end - start );             // nelements
+	    sgWriteUShort( fp, 2 );		         // nproperties
+	    sgWriteUShort( fp, end - start );             // nelements
 
 	    sgWriteChar( fp, (char)SG_MATERIAL );       // property
 	    sgWriteUInt( fp, material.length() );        // nbytes
@@ -1002,19 +1007,20 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 	    // write fans
 	    for ( i = start; i < end; ++i ) {
 		// nbytes
-		sgWriteUInt( fp, fans_v[i].size() * idx_size * sizeof(short) );
+		sgWriteUInt( fp, fans_v[i].size() * idx_size
+                                 * sizeof(unsigned short) );
 		for ( j = 0; j < (int)fans_v[i].size(); ++j ) {
 		    if ( fans_v.size() ) {
-			sgWriteShort( fp, (short)fans_v[i][j] );
+			sgWriteUShort( fp, (unsigned short)fans_v[i][j] );
 		    }
 		    if ( fans_n.size() ) {
-			sgWriteShort( fp, (short)fans_n[i][j] );
+			sgWriteUShort( fp, (unsigned short)fans_n[i][j] );
 		    }
 		    if ( fans_c.size() ) {
-			sgWriteShort( fp, (short)fans_c[i][j] );
+			sgWriteUShort( fp, (unsigned short)fans_c[i][j] );
 		    }
 		    if ( fans_tc.size() ) {
-			sgWriteShort( fp, (short)fans_tc[i][j] );
+			sgWriteUShort( fp, (unsigned short)fans_tc[i][j] );
 		    }
 		}
 	    }
@@ -1042,7 +1048,6 @@ bool SGBinObject::write_bin( const string& base, const string& name,
 bool SGBinObject::write_ascii( const string& base, const string& name,
 			       const SGBucket& b )
 {
-    Point3D p;
     int i, j;
 
     SGPath file = base + "/" + b.gen_base_path() + "/" + name;
@@ -1084,7 +1089,7 @@ bool SGBinObject::write_ascii( const string& base, const string& name,
     // dump vertex list
     fprintf(fp, "# vertex list\n");
     for ( i = 0; i < (int)wgs84_nodes.size(); ++i ) {
-	p = wgs84_nodes[i] - gbs_center;
+        SGVec3d p = wgs84_nodes[i] - gbs_center;
 	
 	fprintf(fp,  "v %.5f %.5f %.5f\n", p.x(), p.y(), p.z() );
     }
@@ -1092,7 +1097,7 @@ bool SGBinObject::write_ascii( const string& base, const string& name,
 
     fprintf(fp, "# vertex normal list\n");
     for ( i = 0; i < (int)normals.size(); ++i ) {
-	p = normals[i];
+        SGVec3f p = normals[i];
 	fprintf(fp,  "vn %.5f %.5f %.5f\n", p.x(), p.y(), p.z() );
     }
     fprintf(fp, "\n");
@@ -1100,7 +1105,7 @@ bool SGBinObject::write_ascii( const string& base, const string& name,
     // dump texture coordinates
     fprintf(fp, "# texture coordinate list\n");
     for ( i = 0; i < (int)texcoords.size(); ++i ) {
-	p = texcoords[i];
+        SGVec2f p = texcoords[i];
 	fprintf(fp,  "vt %.5f %.5f\n", p.x(), p.y() );
     }
     fprintf(fp, "\n");
@@ -1126,13 +1131,13 @@ bool SGBinObject::write_ascii( const string& base, const string& name,
 	    // make a list of points for the group
 	    point_list group_nodes;
 	    group_nodes.clear();
-	    Point3D bs_center;
+	    SGVec3d bs_center;
 	    double bs_radius = 0;
 	    for ( i = start; i < end; ++i ) {
 		for ( j = 0; j < (int)tris_v[i].size(); ++j ) {
-		    group_nodes.push_back( wgs84_nodes[ tris_v[i][j] ] );
-		    bs_center = sgCalcCenter( group_nodes );
-		    bs_radius = sgCalcBoundingRadius( bs_center, group_nodes );
+                    group_nodes.push_back( Point3D::fromSGVec3(wgs84_nodes[ tris_v[i][j] ]) );
+		    bs_center = sgCalcCenter( group_nodes ).toSGVec3d();
+		    bs_radius = sgCalcBoundingRadius( Point3D::fromSGVec3(bs_center), group_nodes );
 		}
 	    }
 
@@ -1177,13 +1182,13 @@ bool SGBinObject::write_ascii( const string& base, const string& name,
 	    // make a list of points for the group
 	    point_list group_nodes;
 	    group_nodes.clear();
-	    Point3D bs_center;
+	    SGVec3d bs_center;
 	    double bs_radius = 0;
 	    for ( i = start; i < end; ++i ) {
 		for ( j = 0; j < (int)strips_v[i].size(); ++j ) {
-		    group_nodes.push_back( wgs84_nodes[ strips_v[i][j] ] );
-		    bs_center = sgCalcCenter( group_nodes );
-		    bs_radius = sgCalcBoundingRadius( bs_center, group_nodes );
+                    group_nodes.push_back( Point3D::fromSGVec3(wgs84_nodes[ strips_v[i][j] ]) );
+		    bs_center = sgCalcCenter( group_nodes ).toSGVec3d();
+		    bs_radius = sgCalcBoundingRadius( Point3D::fromSGVec3(bs_center), group_nodes );
 		}
 	    }
 
