@@ -17,10 +17,13 @@
 //
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: sg_socket.cxx,v 1.9 2004/11/19 21:44:16 curt Exp $
+// $Id$
 
+#ifdef HAVE_CONFIG_H
+#  include <simgear_config.h>
+#endif
 
 #include <simgear/compiler.h>
 
@@ -28,7 +31,7 @@
 #include <strings.h>
 #endif
 
-#include STL_IOSTREAM
+#include <iostream>
 
 #include <simgear/debug/logstream.hxx>
 
