@@ -1,12 +1,12 @@
 #include <simgear/compiler.h>
 #include <simgear/debug/logstream.hxx>
-#include STL_STRING
-#include STL_IOSTREAM
+#include <string>
+#include <iostream>
 
 #include "sg_socket.hxx"
 
-SG_USING_STD(string);
-SG_USING_STD(cout);
+using std::string;
+using std::cout;
 
 class TcpServer
 {
