@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: sg_path.cxx,v 1.21.2.1 2007-01-17 17:12:12 mfranz Exp $
+// $Id$
 
 
 #include <simgear/compiler.h>
@@ -39,13 +39,8 @@
  * define directory path separators
  */
 
-#if defined( macintosh )
-static const char sgDirPathSep = ':';
-static const char sgDirPathSepBad = '/';
-#else
 static const char sgDirPathSep = '/';
 static const char sgDirPathSepBad = '\\';
-#endif
 
 #if defined( WIN32 ) && !defined(__CYGWIN__)
 static const char sgSearchPathSep = ';';
@@ -165,7 +160,7 @@ string SGPath::base() const {
     }
 }
 
-// get the extention (everything after the final ".")
+// get the extension (everything after the final ".")
 // but make sure no "/" follows the "." character (otherwise it
 // is has to be a directory name containing a ".").
 string SGPath::extension() const {
@@ -193,10 +188,10 @@ bool SGPath::exists() const {
 #endif
 
 
-void SGPath::create_dir( mode_t mode ) {
+int SGPath::create_dir( mode_t mode ) {
     string_list dirlist = sgPathSplit(dir());
     if ( dirlist.empty() )
-        return;
+        return -1;
     string path = dirlist[0];
     string_list path_elements = sgPathBranchSplit(path);
     bool absolute = !path.empty() && path[0] == sgDirPathSep;
@@ -216,19 +211,21 @@ void SGPath::create_dir( mode_t mode ) {
         dir.append(path_elements[i]);
     }
     if ( r == 0 ) {
-        return; // Directory already exists
+        return 0; // Directory already exists
     }
     if ( sgMkDir( dir.c_str(), mode) ) {
         SG_LOG( SG_IO, SG_ALERT, "Error creating directory: " + dir.str() );
-        return;
+        return -2;
     }
-    for(;i < path_elements.size(); i++) {
+    for(; i < path_elements.size(); i++) {
         dir.append(path_elements[i]);
         if ( sgMkDir( dir.c_str(), mode) ) {
             SG_LOG( SG_IO, SG_ALERT, "Error creating directory: " + dir.str() );
-            break;
+            return -2;
         }
     }
+
+    return 0;
 }
 
 string_list sgPathBranchSplit( const string &dirpath ) {
