@@ -18,10 +18,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: strutils.cxx,v 1.3.2.1 2007-11-27 19:27:43 durk Exp $
+// $Id$
 
 #include <ctype.h>
 #include <cstring>
+
 #include "strutils.hxx"
 
 namespace simgear {
@@ -83,7 +84,7 @@ namespace simgear {
 		return split_whitespace( str, maxsplit );
 
 	    vector<string> result;
-	    int n = strlen( sep );
+	    int n = std::strlen( sep );
 	    if (n == 0)
 	    {
 		// Error: empty separator string
@@ -97,7 +98,7 @@ namespace simgear {
 
 	    while (i+n <= len)
 	    {
-		if (s[i] == sep[0] && (n == 1 || memcmp(s+i, sep, n) == 0))
+		if (s[i] == sep[0] && (n == 1 || std::memcmp(s+i, sep, n) == 0))
 		{
 		    result.push_back( str.substr(j,i-j) );
 		    i = j = i + n;
