@@ -18,9 +18,9 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: tabbed_values.cxx,v 1.3.2.1 2007-11-27 19:27:43 durk Exp $
+// $Id$
 
-#include <stdlib.h>
+#include <cstdlib>
 
 #include "tabbed_values.hxx"
 
@@ -88,7 +88,7 @@ double SGTabbedValues::getDoubleAt(const unsigned int offset) const
 		
 	/* this is safe because strtod will stop parsing when it sees an unrecogznied
 	character, which includes tab. */	
-	return strtod(data, NULL);
+	return std::strtod(data, NULL);
 }
 
 long SGTabbedValues::getLongAt(const unsigned int offset) const
@@ -97,5 +97,5 @@ long SGTabbedValues::getLongAt(const unsigned int offset) const
 	if (!data || (*data == '\t'))
 		return 0;
 
-	return strtol(data, NULL, 0);
+	return std::strtol(data, NULL, 0);
 }
