@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: texcoord.cxx,v 1.6 2006-03-08 18:16:08 mfranz Exp $
+// $Id$
 
 
 /* The following is an explanation of our somewhat conveluted and
@@ -145,12 +145,12 @@ enter this in the official comments in case I forget again. :-)
 
 #include <simgear/compiler.h>
 
-// #include STL_IOSTREAM
+// #include <iostream>
 
 #include "texcoord.hxx"
 
-// SG_USING_STD(cout);
-// SG_USING_STD(endl);
+// using std::cout;
+// using std::endl;
 
 
 #define FG_STANDARD_TEXTURE_DIMENSION 1000.0 // meters
@@ -176,13 +176,19 @@ static inline Point3D basic_tex_coord( const Point3D& p,
 // calculate "none stretching" texture coordinates
 point_list sgCalcTexCoords( const SGBucket& b, const point_list& geod_nodes,
 			    const int_list& fan, double scale )
+{
+    return sgCalcTexCoords(b.get_center_lat(), geod_nodes, fan, scale);
+}
+
+point_list sgCalcTexCoords( double centerLat, const point_list& geod_nodes,
+			    const int_list& fan, double scale )
 {
     // cout << "calculating texture coordinates for a specific fan of size = "
     //      << fan.size() << endl;
 
     // calculate perimeter based on center of this degree (not center
     // of bucket)
-    double clat = (int)b.get_center_lat();
+    double clat = (int)centerLat;
     if ( clat > 0 ) {
 	clat = (int)clat + 0.5;
     } else {
