@@ -15,20 +15,15 @@
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Library General Public License for more details.
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the
-// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-// Boston, MA  02111-1307, USA.
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: zfstream.cxx,v 1.1.1.1 2002/09/07 02:58:19 curt Exp $
+// $Id$
 
 #include <simgear/compiler.h>
 
-#ifdef SG_HAVE_STD_INCLUDES
-# include <cerrno>
-#else
-# include <errno.h>
-#endif
+#include <cerrno>
 #include <memory.h>
 #include <stdio.h>
 
@@ -39,13 +34,9 @@
 // Allocate memory for 'get' buffer and zero all buffer pointers.
 //
 gzfilebuf::gzfilebuf()
-    : streambuf(),
+    : std::streambuf(),
       file(NULL),
-#if defined( __MWERKS__ ) || __GNUC__ > 2
       mode(ios_openmode(0)),
-#else
-      mode(0),
-#endif
       own_file_descriptor(false),
       ibuf_size(0),
       ibuffer(0)
@@ -175,10 +166,10 @@ gzfilebuf::close()
 // }
 
 
-streampos
-gzfilebuf::seekoff( streamoff, ios_seekdir, int )
+std::streampos
+gzfilebuf::seekoff( std::streamoff, ios_seekdir, int )
 {
-    return streampos(EOF);
+    return std::streampos(EOF);
 }
 
 gzfilebuf::int_type
