@@ -4,43 +4,40 @@
 //
 // See props.html for documentation [replace with URL when available].
 //
-// $Id: props.cxx,v 1.26.2.10 2007-10-13 14:05:48 durk Exp $
+// $Id$
 
 #include "props.hxx"
 
 #include <algorithm>
+
 #include <sstream>
 #include <stdio.h>
 #include <string.h>
 
 #if PROPS_STANDALONE
-
 #include <iostream>
-using std::cerr;
-using std::endl;
-using std::find;
-using std::sort;
-using std::vector;
-using std::stringstream;
-
 #else
 
 #include <simgear/compiler.h>
 #include <simgear/debug/logstream.hxx>
 
-SG_USING_STD(sort);
-SG_USING_STD(find);
-SG_USING_STD(vector);
-SG_USING_STD(stringstream);
-
 #if ( _MSC_VER == 1200 )
 // MSVC 6 is buggy, and needs something strange here
-SG_USING_STD(vector<SGPropertyNode_ptr>);
-SG_USING_STD(vector<SGPropertyChangeListener *>);
-SG_USING_STD(vector<SGPropertyNode *>);
+using std::vector<SGPropertyNode_ptr>;
+using std::vector<SGPropertyChangeListener *>;
+using std::vector<SGPropertyNode *>;
+#endif
 #endif
 
+#if PROPS_STANDALONE
+using std::cerr;
 #endif
+using std::endl;
+using std::find;
+using std::sort;
+using std::string;
+using std::vector;
+using std::stringstream;
 
 
 
@@ -480,6 +477,7 @@ SGPropertyNode::clearValue ()
   case NONE:
     break;
   case ALIAS:
+    put(_value.alias);
     _value.alias = 0;
     break;
   case BOOL:
@@ -627,7 +625,7 @@ SGPropertyNode::trace_read () const
  * Last used attribute
  * Update as needed when enum Attribute is changed
  */
-const int SGPropertyNode::LAST_USED_ATTRIBUTE = TRACE_WRITE;
+const int SGPropertyNode::LAST_USED_ATTRIBUTE = USERARCHIVE;
 
 /**
  * Default constructor: always creates a root node.
@@ -664,6 +662,7 @@ SGPropertyNode::SGPropertyNode (const SGPropertyNode &node)
     break;
   case ALIAS:
     _value.alias = node._value.alias;
+    get(_value.alias);
     _tied = false;
     break;
   case BOOL:
@@ -779,6 +778,7 @@ SGPropertyNode::alias (SGPropertyNode * target)
   if (target == 0 || _type == ALIAS || _tied)
     return false;
   clearValue();
+  get(target);
   _value.alias = target;
   _type = ALIAS;
   return true;
@@ -803,8 +803,7 @@ SGPropertyNode::unalias ()
 {
   if (_type != ALIAS)
     return false;
-  _type = NONE;
-  _value.alias = 0;
+  clearValue();
   return true;
 }
 
@@ -1306,7 +1305,7 @@ SGPropertyNode::setIntValue (int value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    snprintf(buf, 128, "%d", value);
+    sprintf(buf, "%d", value);
     result = set_string(buf);
     break;
   }
@@ -1357,7 +1356,7 @@ SGPropertyNode::setLongValue (long value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    snprintf(buf, 128, "%ld", value);
+    sprintf(buf, "%ld", value);
     result = set_string(buf);
     break;
   }
@@ -1408,13 +1407,7 @@ SGPropertyNode::setFloatValue (float value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    if (snprintf(buf, 128, "%f", value) >= 128) {
-#if PROPS_STANDALONE
-         cerr << "caught buffer overflow in " << SG_ORIGIN << endl;
-#else
-         SG_LOG(SG_GENERAL, SG_ALERT, "caught buffer overflow in " << SG_ORIGIN);
-#endif
-    }
+    sprintf(buf, "%f", value);
     result = set_string(buf);
     break;
   }
@@ -1465,14 +1458,7 @@ SGPropertyNode::setDoubleValue (double value)
   case STRING:
   case UNSPECIFIED: {
     char buf[128];
-    if (snprintf(buf, 128, "%f", value) >= 128) {
-#if PROPS_STANDALONE
-         cerr << "caught buffer overflow in " << SG_ORIGIN << endl;
-#else
-         SG_LOG(SG_GENERAL, SG_ALERT, "caught buffer overflow in " << SG_ORIGIN);
-#endif
-
-    }
+    sprintf(buf, "%f", value);
     result = set_string(buf);
     break;
   }
@@ -2349,9 +2335,8 @@ SGPropertyNode::hash_table::hashcode (const char * key)
 
 SGPropertyChangeListener::~SGPropertyChangeListener ()
 {
-  for (int i = _properties.size() - 1; i >= 0; i--) {
+  for (int i = _properties.size() - 1; i >= 0; i--)
     _properties[i]->removeChangeListener(this);
-    }
 }
 
 void
@@ -2385,9 +2370,8 @@ SGPropertyChangeListener::unregister_property (SGPropertyNode * node)
 {
   vector<SGPropertyNode *>::iterator it =
     find(_properties.begin(), _properties.end(), node);
-  if (it != _properties.end()) {
+  if (it != _properties.end())
     _properties.erase(it);
-  }
 }
 
 
