@@ -5,7 +5,7 @@
  *
  * See props.html for documentation [replace with URL when available].
  *
- * $Id: props_io.cxx,v 1.14.2.3 2007-10-10 12:45:51 mfranz Exp $
+ * $Id$
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,19 +24,22 @@
 #include "props.hxx"
 #include "props_io.hxx"
 
-#include STL_IOSTREAM
-#include STL_FSTREAM
-#include STL_STRING
+#include <iostream>
+#include <fstream>
+#include <string>
+#include <cstring>		// strcmp()
 #include <vector>
 #include <map>
 
-SG_USING_STD(istream);
-SG_USING_STD(ifstream);
-SG_USING_STD(ostream);
-SG_USING_STD(ofstream);
-SG_USING_STD(string);
-SG_USING_STD(vector);
-SG_USING_STD(map);
+using std::istream;
+using std::ifstream;
+using std::ostream;
+using std::ofstream;
+using std::string;
+using std::vector;
+using std::map;
+
+using std::endl;
 
 #define DEFAULT_MODE (SGPropertyNode::READ|SGPropertyNode::WRITE)
 
@@ -73,22 +76,23 @@ private:
 
   struct State
   {
-    State () : node(0), type(""), mode(DEFAULT_MODE) {}
-    State (SGPropertyNode * _node, const char * _type, int _mode)
-      : node(_node), type(_type), mode(_mode) {}
+    State () : node(0), type(""), mode(DEFAULT_MODE), omit(false) {}
+    State (SGPropertyNode * _node, const char * _type, int _mode, bool _omit)
+      : node(_node), type(_type), mode(_mode), omit(_omit) {}
     SGPropertyNode * node;
     string type;
     int mode;
+    bool omit;
     map<string,int> counters;
   };
 
   State &state () { return _state_stack[_state_stack.size() - 1]; }
 
-  void push_state (SGPropertyNode * node, const char * type, int mode) {
+  void push_state (SGPropertyNode * node, const char * type, int mode, bool omit = false) {
     if (type == 0)
-      _state_stack.push_back(State(node, "unspecified", mode));
+      _state_stack.push_back(State(node, "unspecified", mode, omit));
     else
-      _state_stack.push_back(State(node, type, mode));
+      _state_stack.push_back(State(node, type, mode, omit));
     _level++;
     _data = "";
   }
@@ -101,6 +105,7 @@ private:
   int _default_mode;
   string _data;
   SGPropertyNode * _root;
+  SGPropertyNode null;
   int _level;
   vector<State> _state_stack;
   string _base;
@@ -187,6 +192,11 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
 
 				// Got the index, so grab the node.
     SGPropertyNode * node = st.node->getChild(name, index, true);
+    if (!node->getAttribute(SGPropertyNode::WRITE)) {
+      SG_LOG(SG_INPUT, SG_ALERT, "Not overwriting write-protected property "
+          << node->getPath(true));
+      node = &null;
+    }
 
 				// Get the access-mode attributes,
 				// but don't set yet (in case they
@@ -220,6 +230,7 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
     }
 
 				// Check for an include.
+    bool omit = false;
     attval = atts.getValue("include");
     if (attval != 0) {
       SGPath path(SGPath(_base).dir());
@@ -230,26 +241,14 @@ PropsVisitor::startElement (const char * name, const XMLAttributes &atts)
 	setException(e);
       }
 
-      const char *omit = atts.getValue("omit-node");
-      if (omit && !strcmp(omit, "y")) {
-        int nChildren = node->nChildren();
-        for (int i = 0; i < nChildren; i++) {
-          SGPropertyNode *src = node->getChild(i);
-          const char *name = src->getName();
-          int index = st.counters[name];
-          st.counters[name]++;
-          SGPropertyNode *dst = st.node->getChild(name, index, true);
-          copyProperties(src, dst);
-        }
-        st.node->removeChild(node->getName(), node->getIndex(), false);
-        node = st.node;
-      }
+      attval = atts.getValue("omit-node");
+      omit = checkFlag(attval, false);
     }
 
     const char *type = atts.getValue("type");
     if (type)
       node->clearValue();
-    push_state(node, type, mode);
+    push_state(node, type, mode, omit);
   }
 }
 
@@ -299,6 +298,19 @@ PropsVisitor::endElement (const char * name)
 				// assigned.
   st.node->setAttributes(st.mode);
 
+  if (st.omit) {
+    State &parent = _state_stack[_state_stack.size() - 2];
+    int nChildren = st.node->nChildren();
+    for (int i = 0; i < nChildren; i++) {
+      SGPropertyNode *src = st.node->getChild(i);
+      const char *name = src->getName();
+      int index = parent.counters[name];
+      parent.counters[name]++;
+      SGPropertyNode *dst = parent.node->getChild(name, index, true);
+      copyProperties(src, dst);
+    }
+    parent.node->removeChild(st.node->getName(), st.node->getIndex(), false);
+  }
   pop_state();
 }
 
