@@ -1,12 +1,12 @@
 /**
- * $Id: persparam.cxx,v 1.1 2006-08-24 22:46:40 fredb Exp $
+ * $Id$
  */
 
 #include <simgear/props/props.hxx>
 #include "persparam.hxx"
 
 template <> double
-SGPersonalityParameter<double>::getNodeValue( SGPropertyNode *props,
+SGPersonalityParameter<double>::getNodeValue( const SGPropertyNode *props,
                                               const char *name,
                                               double defval ) const
 {
