@@ -1,3 +1,7 @@
+#include <simgear/compiler.h>
+
+#include <osg/GL>
+
 class GlBitmap
 {
 public:
