@@ -46,16 +46,11 @@
  * Nov. 2005, Use the simgear logging facility, Erik Hofman
  * Mar. 2006, Add MAC OS X support, Alexander Powell
  */
-#include <cstring>
 
 #ifdef HAVE_CONFIG_H
 #  include <simgear_config.h>
 #endif
 
-#ifdef HAVE_WINDOWS_H
-#  include <windows.h>
-#endif
-
 #include <simgear/compiler.h>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/screen/extensions.hxx>
@@ -66,6 +61,8 @@
 #include <assert.h>
 #include <stdarg.h>
 
+#include <cstring>
+
 #ifdef _WIN32
 #pragma comment(lib, "gdi32.lib") // required for GetPixelFormat()
 #endif
@@ -299,7 +296,7 @@ void _wglGetLastError()
 * @fn PrintExtensionError( char* strMsg, ... )
 * @brief Prints an error about missing OpenGL extensions.
 */ 
-void PrintExtensionError( char* strMsg, ... )
+void PrintExtensionError( const char* strMsg, ... )
 {
     SG_LOG(SG_GL, SG_ALERT, 
             "Error: RenderTexture requires the following unsupported "
@@ -499,7 +496,7 @@ bool RenderTexture::Initialize(int width, int height,
         fprintf(stderr, "Couldn't get current context!");
    
         CGLPixelFormatObj pixFormat = NULL;
-        long int iNumFormats;
+        GLint iNumFormats;
         CGLError error;
 
         // Copy the _pixelFormatAttribs into another array to fix
@@ -515,7 +512,7 @@ bool RenderTexture::Initialize(int width, int height,
         }
 
         if (error =
-           CGLChoosePixelFormat(&pixFormatAttribs[0], &pixFormat, (GLint *)&iNumFormats))
+           CGLChoosePixelFormat(&pixFormatAttribs[0], &pixFormat, &iNumFormats))
         {
             fprintf(stderr,
                     "RenderTexture Error: CGLChoosePixelFormat() failed.\n");
@@ -556,8 +553,8 @@ bool RenderTexture::Initialize(int width, int height,
            return false;
        }
 
-       long screen;
-       if (error = CGLGetVirtualScreen(CGLGetCurrentContext(), (GLint *)&screen))
+       GLint screen;
+       if (error = CGLGetVirtualScreen(CGLGetCurrentContext(), &screen))
        {
            _cglCheckError(error);
            return false;
@@ -1336,7 +1333,7 @@ void RenderTexture::_ParseModeString(const char *modeString,
         tokens.push_back(buf);
         buf = strtok(NULL, " ");
     }
-    free (mode);
+    free(mode);
     for (unsigned int i = 0; i < tokens.size(); i++)
     {
         string token = tokens[i];
@@ -2056,7 +2053,6 @@ void RenderTexture::_ParseModeString(const char *modeString,
                "Linux or MacOS");
 #endif 
     }
-//////////////////////////////////////////// END OF Function
 }
 
 //---------------------------------------------------------------------------
