@@ -16,7 +16,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: screen-dump.cxx,v 1.7.2.1 2007-06-14 20:20:59 mfranz Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -37,7 +37,7 @@
 
 #include <simgear/compiler.h>
 
-#include SG_GL_H
+#include <osg/GL>
 
 #include "screen-dump.hxx"
 
