@@ -18,18 +18,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: serial.cxx,v 1.11 2006-08-26 14:02:17 curt Exp $
+// $Id$
 
 
 #include <simgear/compiler.h>
 
-#include STL_IOSTREAM
-
-#ifdef SG_HAVE_STD_INCLUDE
-#  include <cerrno>
-#else
-#  include <errno.h>
-#endif
+#include <iostream>
+#include <cerrno>
 
 #if !defined( WIN32 ) || defined( __CYGWIN__) || defined( __CYGWIN32__ )
 #  include <termios.h>
