@@ -21,7 +21,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: soundmgr_openal.cxx,v 1.25.2.1 2007-07-07 18:37:58 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <simgear_config.h>
@@ -68,7 +68,7 @@ extern "C" int isinf (double);
 #endif
 
 
-#include STL_IOSTREAM
+#include <iostream>
 
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/sg_path.hxx>
