@@ -19,7 +19,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: xmlsound.cxx,v 1.19.2.3 2007-07-07 18:50:01 mfranz Exp $
+// $Id$
 
 #ifdef HAVE_CONFIG_H
 #  include <simgear_config.h>
@@ -47,7 +47,7 @@ static double _snd_log(double v)   { return log(fabs(v)); }
 // static double _snd_pow3(double v)  { return v*v*v; }
 
 static const struct {
-	char *name;
+	const char *name;
 	double (*fn)(double);
 } __sound_fn[] = {
 //	{"lin", _snd_lin},
@@ -70,6 +70,7 @@ SGXmlSound::SGXmlSound()
     _prev_value(0),
     _dt_play(0.0),
     _dt_stop(0.0),
+    _delay(0.0),
     _stopping(0.0)
 {
 }
@@ -120,12 +121,14 @@ SGXmlSound::init(SGPropertyNode *root, SGPropertyNode *node, SGSoundMgr *sndmgr,
       SG_LOG(SG_GENERAL, SG_WARN,
              "  Neither a condition nor a property specified");
 
+   _delay = node->getDoubleValue("delay-sec", 0.0);
+
    //
    // set volume properties
    //
    unsigned int i;
    float v = 0.0;
-   vector<SGPropertyNode_ptr> kids = node->getChildren("volume");
+   std::vector<SGPropertyNode_ptr> kids = node->getChildren("volume");
    for (i = 0; (i < kids.size()) && (i < SGXmlSound::MAXPROP); i++) {
       _snd_prop volume = {NULL, NULL, NULL, 1.0, 0.0, 0.0, 0.0, false};
 
@@ -326,8 +329,7 @@ SGXmlSound::update (double dt)
    }
 
    //
-   // If the mode is ONCE and the sound is still playing,
-   //  we have nothing to do anymore.
+   // mode is ONCE and the sound is still playing?
    //
    if (_active && (_mode == SGXmlSound::ONCE)) {
 
@@ -338,16 +340,19 @@ SGXmlSound::update (double dt)
          _dt_play += dt;
       }
 
-      return;
+   } else {
+
+      //
+      // Update the playing time, cache the current value and
+      // clear the delay timer.
+      //
+      _dt_play += dt;
+      _prev_value = curr_value;
+      _stopping = 0.0;
    }
 
-   //
-   // Update the playing time, cache the current value and
-   // clear the delay timer.
-   //
-   _dt_play += dt;
-   _prev_value = curr_value;
-   _stopping = 0.0;
+   if (_dt_play < _delay)
+      return;
 
    //
    // Update the volume
