@@ -2,9 +2,12 @@
 // Started Spring 2001 by David Megginson, david@megginson.com
 // This code is released into the Public Domain.
 //
-// $Id: commands.cxx,v 1.1 2003-09-24 17:19:23 ehofman Exp $
+// $Id$
 
+#include <memory>
 #include <simgear/props/props_io.hxx>
+#include <simgear/threads/SGThread.hxx>
+#include <simgear/threads/SGGuard.hxx>
 
 #include "commands.hxx"
 
@@ -25,6 +28,22 @@ SGCommandMgr::~SGCommandMgr ()
   // no-op
 }
 
+SGCommandMgr*
+SGCommandMgr::instance()
+{
+  static std::auto_ptr<SGCommandMgr> mgr;
+  if (mgr.get())
+    return mgr.get();
+
+  static SGMutex lock;
+  SGGuard<SGMutex> guard(lock);
+  if (mgr.get())
+    return mgr.get();
+
+  mgr = std::auto_ptr<SGCommandMgr>(new SGCommandMgr);
+  return mgr.get();
+}
+
 void
 SGCommandMgr::addCommand (const string &name, command_t command)
 {
