@@ -2,13 +2,12 @@
 // Started Summer 2001 by David Megginson, david@megginson.com
 // This code is released into the Public Domain.
 //
-// $Id: exception.cxx,v 1.3.2.2 2007-10-13 13:51:51 durk Exp $
+// $Id$
 
 
 #include "exception.hxx"
 #include <stdio.h>
 
-
 
 ////////////////////////////////////////////////////////////////////////
 // Implementation of sg_location class.
@@ -93,13 +92,13 @@ sg_location::asString () const
       out += ",\n";
   }
   if (_line != -1) {
-    snprintf(buf, 128, "line %d", _line);
+    sprintf(buf, "line %d", _line);
     out += buf;
     if (_column != -1)
       out += ", ";
   }
   if (_column != -1) {
-    snprintf(buf, 128, "column %d", _column);
+    sprintf(buf, "column %d", _column);
     out += buf;
   }
   return out;
