@@ -18,7 +18,7 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
-// $Id: sg_time.cxx,v 1.10 2006-03-08 18:16:10 mfranz Exp $
+// $Id$
 
 
 #ifdef HAVE_CONFIG_H
@@ -29,15 +29,9 @@
 
 #include <errno.h>		// for errno
 
-#ifdef SG_HAVE_STD_INCLUDES
-#  include <cstdio>
-#  include <cstdlib>
-#  include <ctime>
-#else
-#  include <math.h>
-#  include <stdio.h>
-#  include <stdlib.h>
-#endif
+#include <cstdio>
+#include <cstdlib>
+#include <ctime>
 
 #include <string>
 
